#!/bin/sh

# luna firmware upgrade  script
# $1 image destination (0 or 1) 
# Kernel and root file system images are assumed to be located at the same directory named uImage and rootfs respectively
# ToDo: use arugements to refer to kernel/rootfs location.

########### G3 ##########################################################
### uImage : uImage
### rootfs : rootfs
#########################################################################
k_img="uImage"
r_img="rootfs"
img_ver="version"
md5_cmp="md5.txt"
md5_cmd="/bin/md5sum"
#md5 run-time result
md5_tmp="md5_tmp"
md5_rt_result="md5_rt_result.txt"
new_fw_ver="new_fw_ver.txt"
cur_fw_ver="cur_fw_ver.txt"
env_sw_ver="env_sw_ver.txt"
hw_ver_file="hw_ver"
skip_hwver_check="/tmp/skip_hwver_check"
img_k_size_dec=0
img_r_size_dec=0
img_tar=img.tar

# For YueMe framework
framework_img="framework.img"
framework_sh="framework.sh"
framework_upgraded=0

#arg1="$1"
sw_active=`nv getenv sw_active | cut -d '=' -f2`
sw_commit=`nv getenv sw_commit | cut -d '=' -f2`
if [ "$sw_commit" = "0" ]; then 
		if [ "$sw_active" = "0" ]; then 
				arg1=1
		else
				arg1=0
		fi
else
		if [ "$sw_active" = "1" ]; then 
				arg1=0
		else
				arg1=1
		fi
fi		
arg2="$1"

echo_everyone() {
	file_list=$(ls /dev/pts)
	if [ -n file_list ]; then
		for f in $file_list
		do
			echo $1 > /dev/pts/$f
		done
	fi
	echo $1 > /dev/console
}

set_upgrade_status() {
	retval=1
	if [ "$1" -eq "0" ] || [ "$1" -eq "1" ]; then
		/bin/setupgradestatus $1
		retval=$?
	fi
	return $retval
}

fail_exit() {
	set_upgrade_status 0
	if [ -n "$2" ]; then
		echo "UPGRADE FAIL: $2"
	fi
	exit $1
}

update_framework() {

	if [ "`tar -xOf $2 $img_tar | tar -t $framework_sh`" = "$framework_sh" ] && [ "`tar -xOf $2 $img_tar | tar -t $framework_img`" = "$framework_img" ]; then
			echo "Updaing framework from $2"
			tar -xOf $2 $img_tar | tar -x $framework_sh
			grep $framework_sh $md5_cmp > $md5_tmp
			$md5_cmd $framework_sh > $md5_rt_result
			diff $md5_rt_result $md5_tmp

			if [ $? != 0 ]; then 
					fail_exit 1 "$framework_sh md5_sum inconsistent, aborted image updating !"
			fi

			# Run firmware upgrade script extracted from image tar ball
			sh $framework_sh $2
			framework_upgraded=1
	fi

	if [ "`tar -xOf $2 $img_tar | tar -t $k_img`" = '' ] && [ $framework_upgraded = 1 ]; then
			fail_exit 2 "No uImage for upgrading, skip"
	fi
}

do_hwver_check() {
	if [ -f $skip_hwver_check ]; then
			echo "Skip HW_VER check!!"
	else
			img_hw_ver=`tar -xOf $2 $img_tar | tar -x $hw_ver_file -O`
			mib_hw_ver=`flash get HW_HWVER | sed s/HW_HWVER=//g`
			if [ "$img_hw_ver" = "skip" ]; then
					echo "skip HW_VER check!!"
			else
					echo "img_hw_ver=$img_hw_ver mib_hw_ver=$mib_hw_ver"
					if [ "$img_hw_ver" != "$mib_hw_ver" ]; then
							fail_exit 1 "HW_VER $img_hw_ver inconsistent, aborted image updating !"
					fi
			fi
	fi
}

do_extract_img_md5() {
	# Extract kernel image
	if [ -f ./$k_img ]; then
		md5_rt_result=`busybox md5sum ./$k_img | cut -d' ' -f 1`
	elif [ "`tar -xOf $2 $img_tar | tar -t $k_img`" = "$k_img" ]; then
		md5_rt_result=`tar -xOf $2 $img_tar | tar -xO $k_img | busybox md5sum | cut -d' ' -f 1`
	fi

	# Check integrity
	md5_tmp_1=`grep $k_img $md5_cmp | head -n 1| cut -d " " -f1`
#	diff $md5_rt_result $md5_tmp

	if [ "$md5_rt_result" != "$md5_tmp_1" ]; then
		fail_exit 1 "$k_img"" md5_sum inconsistent, aborted image updating !"
	fi

	# Extract rootfs image
	if [ -f ./$r_img ]; then
		md5_rt_result=`busybox md5sum ./$r_img | cut -d' ' -f 1`
	elif [ "`tar -xOf $2 $img_tar | tar -t $r_img`" = "$r_img" ]; then
		md5_rt_result=`tar -xOf $2 $img_tar | tar -xO $r_img | busybox md5sum | cut -d' ' -f 1`
	fi

	# Check integrity
	md5_tmp_1=`grep $r_img $md5_cmp | head -n 1| cut -d " " -f1`
#	diff $md5_rt_result $md5_tmp

	if [ "$md5_rt_result" != "$md5_tmp_1" ]; then
		# rm $r_img
		fail_exit 1 "$r_img"" md5_sum inconsistent, aborted image updating !"
	fi

	echo "Integrity of $k_img & $r_img is okay."
}

do_firware_ver_chk() {
	# Check upgrade firmware's version with current firmware version
#	tar -xf $2 $img_ver
	if ! test -f $img_ver; then
		fail_exit 1 "Firmware version incorrect: no fwu_ver in img.tar !"
	fi

	cat $img_ver > $new_fw_ver
	cat /version > $cur_fw_ver

	#Because we don't have full fw version in fwu_ver, so we cut full version from
	#"V4.0.0-191014 -- Mon Oct 14 15:10:15 CST 2019" to be "V4.0.0-191014"

	short_cur_fw_ver=`cat $cur_fw_ver | cut -d " " -f1`
	echo "Try to upgrade firmware version from `cat $cur_fw_ver`"
	echo "                                  to `cat $new_fw_ver`"
}

do_firmware_space_check() {
# Find out kernel/rootfs mtd partition according to image destination
k_mtd="/dev/"`cat /proc/mtd | grep \"ubi_k"$1"\" | sed 's/:.*$//g'`
r_mtd="/dev/"`cat /proc/mtd | grep \"ubi_r"$1"\" | sed 's/:.*$//g'`
k_mtd_size=`cat /proc/mtd | grep \"ubi_k"$1"\" | sed 's/^.*: //g' | sed 's/ .*$//g'`
r_mtd_size=`cat /proc/mtd | grep \"ubi_r"$1"\" | sed 's/^.*: //g' | sed 's/ .*$//g'`
echo "kernel image is located at $k_mtd"
echo "rootfs image is located at $r_mtd"

if [ -f ./$k_img ]; then
	string="`ls -l | grep $k_img`"
	img_k_size_dec="`expr substr "$string" 34 100 | sed 's/^ *//g' | sed 's/ .*$//g'`"
elif [ "`tar -xOf $2 $img_tar | tar -t $k_img`" = "$k_img" ]; then
	string="`tar -xOf $2 $img_tar | tar -vt $k_img | grep $k_img`"
	img_k_size_dec="`expr substr "$string" 21 87 | sed 's/^ *//g' | sed 's/ .*$//g'`"
fi

mtd_size_dec="`printf %d 0x$k_mtd_size`"
expr "$img_k_size_dec" \< "$mtd_size_dec" > /dev/null
if [ $? != 0 ]; then
	fail_exit 1 "uImage size too big($img_k_size_dec) !"
fi

if [ -f ./$r_img ]; then
	string="`ls -l | grep $r_img`"
	img_r_size_dec="`expr substr "$string" 34 100 | sed 's/^ *//g' | sed 's/ .*$//g'`"
elif [ "`tar -xOf $2 $img_tar | tar -t $r_img`" = "$r_img" ]; then
	string="`tar -xOf $2 $img_tar | tar -vt $r_img | grep $r_img`"
	img_r_size_dec="`expr substr "$string" 21 87 | sed 's/^ *//g' | sed 's/ .*$//g'`"
fi

mtd_size_dec="`printf %d 0x$r_mtd_size`"
expr "$img_r_size_dec" \< "$mtd_size_dec" > /dev/null
if [ $? != 0 ]; then
	fail_exit 1 "rootfs size too big($img_r_size_dec) !"
fi

echo "Both uImage and rootfs size check okay, start updating ..."
}

get_vol_num_from_vol_name() {
	dev_num="$1"
	vol_name="$2"
	info=$(ubinfo -d "${dev_num}" -N "${vol_name}")
	set $info
	echo "/dev/ubi${dev_num}_$3"
}

do_extract_and_update_img() {
	img_num=$1
	tar_name=$2
	vkimg=$(get_vol_num_from_vol_name "0" "ubi_k${img_num}")
	vrimg=$(get_vol_num_from_vol_name "0" "ubi_r${img_num}")
	nv setenv sw_version"$1" "None"

	if [ -f ./$k_img ]; then
		#tar -xf "${tar_name}" $k_img
		ubiupdatevol "$vkimg" "$k_img"
	elif [ "`tar -xOf $2 $img_tar | tar -t $k_img`" = "$k_img" ]; then
		tar -xOf $2 $img_tar | tar -xO $k_img | ubiupdatevol "$vkimg" - --size=$img_k_size_dec
	fi

	if [ -f ./$r_img ]; then
		#tar -xf "${tar_name}" $r_img
		ubiupdatevol "$vrimg" "$r_img"
	elif [ "`tar -xOf $2 $img_tar | tar -t $r_img`" = "$r_img" ]; then
		tar -xOf $2 $img_tar | tar -xO $r_img | ubiupdatevol "$vrimg" - --size=$img_r_size_dec
	fi
}

write_ver_record_and_clean() {
	cat $new_fw_ver | grep CST
	if [ $? = 0 ]; then
		echo `cat $new_fw_ver` | sed 's/ *--.*$//g' > $env_sw_ver
	else
		cat $new_fw_ver > $env_sw_ver
	fi
	# Write image version information 
	if [ -f ./$k_img ] && [ -f ./$r_img ]; then
		nv setenv ubi_k"$1"_crc "`nv crc32 $k_img "$img_k_size_dec"`"
		nv setenv ubi_r"$1"_crc "`nv crc32 $r_img "$img_r_size_dec"`"
	else
		nv setenv ubi_k"$1"_crc "`tar -xOf $2 $img_tar | tar -xO $k_img | nv crc32 stdin "$img_k_size_dec"`"
		nv setenv ubi_r"$1"_crc "`tar -xOf $2 $img_tar | tar -xO $r_img | nv crc32 stdin "$img_r_size_dec"`"
	fi
	nv setenv ubi_k"$1"_size "`printf %x $img_k_size_dec`"
	nv setenv ubi_r"$1"_size "`printf %x $img_r_size_dec`"
	nv setenv sw_version"$1" "`cat $env_sw_ver`"
	nv setenv sw_commit $arg1

	# Clean up temporary files
	rm -f $md5_cmp $md5_tmp $md5_rt_result $img_ver $new_fw_ver $cur_fw_ver $env_sw_ver $k_img $r_img $2

	# Post processing (for future extension consideration)

	echo "UPGRADE: Successfully updated image $1!!"
}

main() {
	set_upgrade_status 1
	if [ "$?" -ne "0" ]; then
		echo_everyone "UPGRADE FAIL: Upgrade is already in progress - can't write flash"
		exit 1
	fi
#	update_framework "$arg1" "$arg2"
#	do_hwver_check "$arg1" "$arg2"
	do_extract_img_md5 "$arg1" "$arg2"
	do_firware_ver_chk "$arg1" "$arg2"
	do_firmware_space_check "$arg1" "$arg2"
	do_extract_and_update_img "$arg1" "$arg2"
	write_ver_record_and_clean "$arg1" "$arg2"
	set_upgrade_status 0
}

sig_handler() {
    fail_exit 1
}

trap sig_handler SIGINT SIGQUIT SIGKILL SIGALRM SIGTERM
main

# Stop this script upon any error
# set -e

